module AdvancedDCL
  module Fig
    def param_init
      @param = Parameter.new(Common.param)
      @param.init("mapangle", [3, Float], "grsmpl", "sgqmpl", 2, 1)
      @param.init("simtrans", [3, Float], "grssim", "sgqsim", 2, 1)
      @param.init("transnum", Fixnum, "grstrn", "sgqtrn", 1, 1)
      @param.init("transnum3d", Fixnum, "scstrn", "scqtrn", 1, 1)
      @param.init("viewport", [4, Float], "grsvpt", "sgqvpt", 2, 1)
      @param.init("viewport3d", [6, Float], "scsvpt", "scqvpt", 2, 1)
      @param.init("window", [4, Numeric], "grswnd", "sgqwnd", 2, 1)
      @param.init("window3d", [6, Numeric], "scswnd", "scqwnd", 2, 1)
      @param.init("l2to3", TrueClass, "sglset", "sglset", 0, 0)
      @param.init("eyepoint", [3, Numeric], "scseye", "scqeye", 2, 1)
      @param.init("objpoint", [3, Numeric], "scsobj", "scqobj", 2, 1)
      @param.init("plane", [3, Numeric], "scspln", "scqpln", 2, 1)
      @param.init("mapwindow", [4, Numeric], "grstxy", "grqtxy", 2, 1)
      @param.init("origin", [4, Numeric], "scsorg", "scqorg", 2, 1)
      @param.init("log3d", [3, Numeric], "scslog", "scqlog", 2, 1)
      @param.init("contactpoint", [3, Float], "umscnt", "umqcnt", 2, 1)
      @param.init("circlewindow", [3, Float], "umscwd", "umqcwd", 2, 1)
      @param.init("inclpoint", [2, [true, Float]], "inclpoint", "", 3, nil)
      @param.init("stlat1", Float, "sgrset", "sgrget", 0, 0)
      @param.init("stlat2", Float, "sgrset", "sgrget", 0, 0)
      
      @param.init("lclip", TrueClass, "sglset", "sglget", 0, 0)
      @param.init("lmiss", TrueClass, "gllset", "gllget", 0, 0)
      @param.init("imiss", Fixnum, "gliset", "gliget", 0, 0)
      @param.init("rmiss", Float, "glrset", "glrget", 0, 0)
      @param.init("rundef", Float, "glrset", "glrget", 0, 0)
      @param.init("iundef", Fixnum, "glrset", "glrget", 0, 0)
      @param.init("ldeg", TrueClass, "sglset", "sglget", 0, 0)

      @ntrsparam = ["lclip", "lmiss", "imiss", "rmiss", "rundef", "iundef", "ldeg"]
      @temp_param = @param.dup
      @backup = @param.dup

      @grstrf = false       # true:called, flase:not called  
      @scstrf = false       # true:called, flase:not called  
    end
    
    private :param_init	
    
    def new
      @grstrf = false
      @scstrf = false
      DCL.grfig
    end
    
#------------------- method for parameters----------------------

    def set(key, val)
      @param[key] = val
      @ntrsparam.each{|i|
	if(i==key)
	  @param.sync
	  return nil
	end
      }
      trfnew
    end

    def settrf
      if(!@grstrf)
	@param.sync
	DCL.uspfit
	DCL.scsprj
	DCL.grstrf
	trfclear
      end
    end

    def setumtrf
      if(!@grstrf)
	@param.sync
	DCL.umpfit
	DCL.scsprj
	DCL.grstrf
	trfclear
      end
    end

    def set3dtrf
      if(!@grstrf)
	@param.sync
	DCL.scsprj
	DCL.scstrf
	trfclear
      end
    end

    def trfclear
      @grstrf = true
      @scstrf = true
    end

    def trfnew
      @grstrf = false
      @scstrf = false
    end

    def short2long(sname)
      DCL.sgtrsl(sname)
    end

    def sname2trn(sname)
      DCL.sgtrsn(sname)
    end

    def long2short(lname)
      DCL.sgtrls(lname)
    end

    def lname2trn(lname)
      DCL.sgtrln(lname)
    end

    def trn2sname(num)
      DCL.sgtrns(num)
    end

    def trn2lname(num)
      DCL.sgtrnl(num)
    end

#----------- draw method -------------

    def pvpr(index)
      DCL.slpvpr(index)
    end

    def pwwr(index)
      DCL.slpwwr(index)
    end

    def pwvr(index)
      DCL.slpwvr(index)
    end

    def pvpc(index, rc)
      DCL.pvpc(index, rc)
    end

    def pwwc(index, rc)
      DCL.pwwc(index, rc)
    end

    def pwvc(index, rc)
      DCL.slpwvc(index, rc)
    end

    private :trfclear
    
    module_function :param_init
    module_function :new
    module_function :set
    module_function :settrf
    module_function :setumtrf
    module_function :trfnew
    module_function :trfclear
    module_function :set3dtrf
    module_function :short2long
    module_function :sname2trn
    module_function :long2short
    module_function :lname2trn
    module_function :trn2sname
    module_function :trn2lname
    module_function :pvpr
    module_function :pwwr
    module_function :pwvr
    module_function :pvpc
    module_function :pwwc
    module_function :pwvc
    
    param_init

#------------------- singleton methods ----------------------

    def @param.inclpoint(point)
      rundef = Fig.rundef
      if(rundef==point[0])
	DCL.usspnt(point[1].length, point[0], point[1])
      else
	DCL.usspnt(point[0].length, point[0], point[1])
      end
    end

    def @temp_param.inclpoint(point)
      if(point[0]==Fig.rundef)
	DCL.usspnt(point[1].length, point[0], point[1])
      else
	DCL.usspnt(point[0].length, point[0], point[1])
      end
    end

    def @backup.inclpoint(point)
      if(point[0]==Fig.rundef)
	DCL.usspnt(point[1].length, point[0], point[1])
      else
	DCL.usspnt(point[0].length, point[0], point[1])
      end
    end
#----------------------------------------------------------

    @param.keys.each{|i|
      eval "def #{i}; @param['#{i}']; end"
      eval "def #{i}=(val); set('#{i}', val); end"
    }
  
    @param.keys.each{|i|
      eval "module_function :#{i}"
      eval "module_function :#{i}="
    }
  end
end
